package com.diagbot.facade;

import com.diagbot.entity.QuestionUsual;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.UsualEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.QuestionUsualService;
import com.diagbot.service.impl.QuestionUsualServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.QuestionUsualVO;
import com.diagbot.vo.UpdateQuetionUsualVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description:
 * @author: wangyu
 * @time: 2018/12/3 17:06
 */
@Component
public class QuestionUsualFacade extends QuestionUsualServiceImpl {
    @Autowired
    private QuestionInfoFacade questionInfoFacade ;
    @Autowired
    private DeptInfoFacade deptInfoFacade;
    @Autowired
    @Qualifier("questionUsualServiceImpl")
    private QuestionUsualService questionUsualService;

    /**
     * 添加常用标签
     * @param questionUsualVO
     * @return
     */
    public Boolean addQuestionUsual(QuestionUsualVO questionUsualVO) {
        List<QuestionUsual> questionUsualList = Lists.newArrayList();
        QuestionUsual questionUsual = new QuestionUsual();
        String userId = "1"/*UserUtils.getCurrentPrincipleID()*/;
        for (int i = 0; i<questionUsualVO.getQuestionId().size(); i++ ) {
            questionUsual = new QuestionUsual();
            questionUsual.setCreator(userId);
            questionUsual.setModifier(userId);
            questionUsual.setGmtCreate(DateUtil.now());
            questionUsual.setGmtModified(DateUtil.now());
            questionUsual.setQuestionId(questionUsualVO.getQuestionId().get(i));
            questionUsual.setDeptId(questionUsualVO.getDeptId());
            questionUsual.setOrderNo(Long.parseLong(String.valueOf(i+1)));
            questionUsual.setUsual(String.valueOf(UsualEnum.Enable.getKey()));
            questionUsualList.add(questionUsual);
        }
        Boolean flag = questionUsualService.saveBatch(questionUsualList);
        if(!flag){
            throw new CommonException(CommonErrorCode.INSERT_DATA_FAILED,
                    "常用症状添加失败");
        }
        return flag;
    }

    /**
     * 修改常用标签
     * @param updateQuetionUsualVO
     * @return
     */
    public Boolean updateQuestionUsual(UpdateQuetionUsualVO updateQuetionUsualVO){
        QuestionUsual questionUsual = new QuestionUsual();
        BeanUtil.copyProperties(updateQuetionUsualVO,questionUsual);
        questionUsual.setGmtModified(DateUtil.now());
        questionUsual.setModifier(UserUtils.getCurrentPrincipleID());
        Boolean flag = this.updateById(questionUsual);
        if(!flag){
            throw new CommonException(CommonErrorCode.FAIL,
                    "常用症状修改失败");
        }
        return flag;
    }

    /**
     * 删除常用标签
     * @param updateQuetionUsualVO
     * @return
     */
    public Boolean deleteQuestionUsual(UpdateQuetionUsualVO updateQuetionUsualVO){
        QuestionUsual questionUsual = new QuestionUsual();
        BeanUtil.copyProperties(updateQuetionUsualVO,questionUsual);
        questionUsual.setIsDeleted(IsDeleteEnum.Y.getKey());
        questionUsual.setGmtModified(DateUtil.now());
        questionUsual.setModifier(UserUtils.getCurrentPrincipleID());
        Boolean flag = this.updateById(questionUsual);
        if(!flag){
            throw new CommonException(CommonErrorCode.FAIL,
                    "常用症状删除失败");
        }
        return flag;
    }

}
